﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using MotorPlantContracts.ViewModels;
using Microsoft.Extensions.Logging;
using MotorPlantDataModels.Enums;
using MotorPlantBusinessLogic.MailWorker;
using System.Xml.Linq;
using MotorPlantContracts.VeiwModels;


namespace MotorPlantBusinessLogic.BusinessLogics
{
	public class OrderLogic : IOrderLogic
	{
		private readonly ILogger _logger;
		private readonly IOrderStorage _orderStorage;
		private readonly AbstractMailWorker _mailWorker;
		private readonly IClientLogic _clientLogic;
		public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, AbstractMailWorker mailWorker, IClientLogic clientLogic)
		{
			_logger = logger;
			_orderStorage = orderStorage;
			_mailWorker = mailWorker;
			_clientLogic = clientLogic;
		}
		public List<OrderViewModel>? ReadList(OrderSearchModel? model)
		{
			_logger.LogInformation("ReadList. ClientId:{ClientId}.Status:{Status}.ImplementerId:{ImplementerId}.DateFrom:{DateFrom}.DateTo:{DateTo}OrderId:{Id}",
				model?.ClientId, model?.Status, model?.ImplementerId, model?.DateFrom, model?.DateTo, model?.Id);
			var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}
		public bool CreateOrder(OrderBindingModel model)
		{
			CheckModel(model);
			if (model.Status != OrderStatus.Неизвестен)
				return false;
			model.Status = OrderStatus.Принят;
			var element = _orderStorage.Insert(model);
			if (element == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			Task.Run(() => _mailWorker.MailSendAsync(new MailSendInfoBindingModel
			{
				MailAddress = element.ClientEmail,
				Subject = $"Изменение статуса заказа номер {element.Id}",
				Text = $"Ваш заказ номер {element.Id} на мотор {element.EngineName} от {element.DateCreate} на сумму {element.Sum} принят."
			}));
			SendOrderStatusMail(element.ClientId, $"Новый заказ создан. Номер заказа #{element.Id}", $"Заказ #{element.Id} от {element.DateCreate} на сумму {element.Sum:0.00} принят");
			return true;
		}

		public bool TakeOrderInWork(OrderBindingModel model)
		{
			return StatusUpdate(model, OrderStatus.Выполняется);
		}

		public bool FinishOrder(OrderBindingModel model)
		{
			return StatusUpdate(model, OrderStatus.Готов);
		}

		public bool DeliveryOrder(OrderBindingModel model)
		{
			return StatusUpdate(model, OrderStatus.Выдан);
		}

		private void CheckModel(OrderBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (model.Count <= 0)
			{
				throw new ArgumentException("Колличество пиццы в заказе не может быть меньше 1", nameof(model.Count));
			}
			if (model.Sum <= 0)
			{
				throw new ArgumentException("Стоимость заказа на может быть меньше 1", nameof(model.Sum));
			}
			if (model.DateImplement.HasValue && model.DateImplement < model.DateCreate)
			{
				throw new ArithmeticException($"Дата выдачи заказа {model.DateImplement} не может быть раньше даты его создания {model.DateCreate}");
			}
			_logger.LogInformation("Engine. EngineId:{EngineId}.Count:{Count}.Sum:{Sum}Id:{Id}",
				model.EngineId, model.Count, model.Sum, model.Id);
		}

		private bool StatusUpdate(OrderBindingModel model, OrderStatus newStatus)
		{
			var viewModel = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
			if (viewModel == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (viewModel.Status + 1 != newStatus)
			{
				_logger.LogWarning("Change status operation failed");
				throw new InvalidOperationException();
			}
			model.Status = newStatus;
			if (model.Status == OrderStatus.Готов)
			{
				model.DateImplement = DateTime.Now;
			}
			else
			{
				model.DateImplement = viewModel.DateImplement;
			}
			if (viewModel.ImplementerId.HasValue)
			{
				model.ImplementerId = viewModel.ImplementerId.Value;
			}
			CheckModel(model, false);
			var result = _orderStorage.Update(model);
			if (result == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			SendOrderStatusMail(result.ClientId, $"Изменен статус заказа #{result.Id}", $"Заказ #{model.Id} изменен статус на {result.Status}");
			return true;
		}

		public OrderViewModel? ReadElement(OrderSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. ClientId:{ClientId}.Status:{Status}.ImplementerId:{ImplementerId}.DateFrom:{DateFrom}.DateTo:{DateTo}OrderId:{Id}",
				model.ClientId, model.Status, model.ImplementerId, model.DateFrom, model.DateTo, model.Id);
			var element = _orderStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}
		private bool SendOrderStatusMail(int clientId, string subject, string text)
		{
			var client = _clientLogic.ReadElement(new() { Id = clientId });
			if (client == null)
			{
				return false;
			}
			_mailWorker.MailSendAsync(new()
			{
				MailAddress = client.Email,
				Subject = subject,
				Text = text
			});
			return true;
		}
	}
}
